require 'kyotocabinet'

module Bender
  class Backend

    attr_reader :config
    attr_reader :log
    attr_reader :host_identity
    attr_reader :store

    def initialize(agent, config, options = {})
      @config = config
      @agent = agent
      @log = options[:log] || ::Tomte.logger
      @host_identity = config[:local_host_identity] || Socket.gethostname

      @store = 
      # create the database object
      db = KyotoCabinet::DB::new
      
      # open the database
      unless db.open(config[:base_config][:local_store] + '.kch', KyotoCabinet::DB::OWRITER | KyotoCabinet::DB::OCREATE)
        raise StandardError.new "Cannot open datanase #{config[:base_config][:local_store] + '.kct'}: %s\n" % db.error
      end

      setup_all_services
    end

    def provision(service = nil, *args)
      service ? do_with_service(service, :provision, *args) : do_with_all_services(:provision, *args)
    end

    def update(service = nil, *args)
      service ? do_with_service(service, :update, *args) : do_with_all_services(:update, *args)
    end

    def maintenance(service = nil, *args)
      service ? do_with_service(service, :maintenance, *args) : do_with_all_services(:maintenance, *args)
    end

    def tasks(service)
      do_with_service(service, :tasks)
    end

    def do(service, task, *args)
      svc = services[service.to_sym]
      if svc.respond_to?(task.to_sym)
        do_with_service(service, task, *args)
      else
        log.error "- service #{service} has no task '#{task}'"
      end
    end

    def do_shutdown
      @store.close
    end

    def shutdown!
      do_shutdown
      @agent.shutdown!
    end

    private
    #######

    def services
      @services ||= {}
    end

    def do_with_all_services(method, *args)
      services.each_key { |svc| do_with_service(svc, method, *args)}
    end

    def do_with_service(service, method, *args)

      svc = services[service.to_sym]
      raise "- No such service '#{service}'" unless svc

      svc.banner
      svc.send(method.to_sym, *args)
    end



    def setup_all_services
      return if !config[:services] && config[:services].blank?
      config[:services].each do |name, svc_config|
        services[name] = determine_service_plugin(svc_config[:service], svc_config[:driver]).new(name, host_identity, svc_config, self)
      end
    end


    def determine_service_plugin(service, driver = nil)
      plugins_path = "#{::Tomte.config.core[:application_lib]}/bender/services/"
      require File.join(plugins_path, service)
      "Bender::Services::#{service.camelize}".constantize
    end

  end
end