module Bender
  module Services
    class LawFilters < Bender::ServiceImplementation

      def provision
        log.info "- provisioning for law filters"

      end

      def update(host = nil)
        buffer = "server:\n"
        list = get_all_ordinances
        list.each do |o|
          mode_address =
            case o.mode.to_sym
              when :not_existing
              when :loopback
                "IN A 127.0.0.1"
              when :redirect
                "IN A #{redirect_ip}"
              else
                "IN A 127.0.0.1"
              end

          o.domains.each do |d|
            buffer += %Q[\tlocal-data: "#{d.name} #{mode_address}"\n]
          end
        end
        output_configuration buffer
      end

      def maintenance(host = nil)
        log.info "- doing maintenance law filters"
      end

      private

      def redirect_ip
        Ygg::LawFilter::Ordinance.redirect_ip
      end

      def get_all_ordinances
        Ygg::LawFilter::Ordinance.to_provision
      end

      def output_configuration(buffer)
        write_file svc_config[:filter_file], buffer
      end

      def write_file(file, data)
        fileout = [file, 'new'].join('.')

        log.info "Writing new configuration to: #{fileout}"
        File.open(fileout, "w+") do |f|
          f.write(data)
        end

        if dry_run?
          log.info "DRYRUN: should move '#{fileout}' to: '#{file}'."
        else
          FileUtils.cp(fileout, file)
          FileUtils.rm(fileout)
        end
      end


    end
  end
end
