
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

require 'bender'

module Tomte::Agent
  class Bender < Tomte::Agent::Sync

    DEFAULT_BENDER_OPTIONS = {
        :api_timeout => 240,
        :api_url => 'https://hel.intercom.it',
        :api_identity => 'provisioning-agent@intercom.it',
        :base_config => { :local_store => '/tmp/bender-store' }
    }

    read_configuration 'bender_defaults.yml', :path => :application
    read_configuration 'bender_defaults.yml', :path => :system
    read_configuration 'bender.yml', :path => :application
    read_configuration 'bender.yml', :path => :system

    caption   "Bender"
    help_text "This is the new Hel/Tomte based Bender for simple services."

    cli_commands :services    => "          List local services",
                 :provision   => "[service] Provisions a new service.",
                 :update      => "[service] Updates the service on this instance.",
                 :maintenance => "          Perform maintenance on all the services.",
                 :do          => "<service> <task> [args...]",
                 :tasks       => "<service>"

    base_configuration :bender, DEFAULT_BENDER_OPTIONS

    additional_options "Bender API options"  do |options, opts, config|

      opts.on("--api-url URL", "The url for the HEL api [#{config[:bender][:api_url]}]") do |api_url|
        options[:bender][:api_url] = api_url
      end

      opts.on("--api-identity IDENTITY", "The identity to connect to the API [#{config[:bender][:api_identity]}]") do |identity|
        options[:bender][:api_identity] = identity
      end

      opts.on("--api-secret SECRET", "The secret to use when connecting to the API") do |secret|
        options[:bender][:api_secret] = secret
      end

      opts.on("--force", "Perform action even if the subsystem is clean") do |force|
        options[:bender][:force] = force
      end

      opts.on("--api-timeout TIMEOUT", "Timeout for HTTP requests.") do |timeout|
        options[:bender][:api_timeout] = timeout
      end
    end


    def init(options = {})
      ::Bender::Resources.setup(config.bender[:api_url], :identity => config.bender[:api_identity],
                                                         :secret => config.bender[:api_secret],
                                                         :timeout => config.bender[:api_timeout])

      @backend = ::Bender::Backend.new(self, config.bender)
    end

    def run

      log.error("What do you want to do, today?")
      shutdown!
    end

    def terminate
      @backend.do_shutdown if @backend
    end

    def services
      ap config.bender[:services]
    end

    def tasks(service)
      @backend.tasks(service)
    end

    def do(service, task, *args)
      @backend.do(service, task, *args)
    end

    def provision(service = nil, *args)
      @backend.provision(service, *args)
    end

    def update(service = nil, *args)
      @backend.update(service, *args)
    end

    def maintenance(service = nil, *args)
      @backend.maintenance(service, *args)
    end
  end
end

